import { BaseEvent } from '#application/base/baseEvent'
import ZoneManager from '#managers/zoneManager'

export default class DisconnectEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('disconnect', this.handleEvent.bind(this))
  }

  private async handleEvent(data: any): Promise<void> {
    try {
      if (!this.socket.userId) {
        this.logger.info('User disconnected but had no user set')
        return
      }

      this.io.emit('user:disconnect', this.socket.userId)

      const zoneCharacter = ZoneManager.getCharacterById(this.socket.characterId!)
      if (!zoneCharacter) {
        this.logger.info('User disconnected but had no character set')
        return
      }

      const character = zoneCharacter.character

      // Save character position and remove from zone
      zoneCharacter.isMoving = false
      await zoneCharacter.savePosition()
      ZoneManager.removeCharacter(this.socket.characterId!)

      this.logger.info('User disconnected along with their character')

      // Inform other clients that the character has left
      this.io.in(character.zone!.id.toString()).emit('zone:character:leave', character.id)
      this.io.emit('character:disconnect', character.id)
    } catch (error: any) {
      this.logger.error('disconnect error: ' + error.message)
    }
  }
}
