import { BaseEvent } from '#application/base/baseEvent'
import { CharacterHair } from '#entities/characterHair'
import characterHairRepository from '#repositories/characterHairRepository'
import characterRepository from '#repositories/characterRepository'

interface IPayload {}

export default class characterHairListEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:characterHair:list', this.handleEvent.bind(this))
  }

  private async handleEvent(data: IPayload, callback: (response: CharacterHair[]) => void): Promise<void> {
    const character = await characterRepository.getById(this.socket.characterId as number)
    if (!character) {
      this.logger.error('gm:characterHair:list error', 'Character not found')
      return callback([])
    }

    if (character.role !== 'gm') {
      this.logger.info(`User ${character.id} tried to list character hair but is not a game master.`)
      return callback([])
    }

    // get all objects
    const items = await characterHairRepository.getAll()
    callback(items)
  }
}
